#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Office 365 Address Lists.

Primary module.
"""

from __future__ import absolute_import, print_function
import logging
from xml.etree import ElementTree
import sys
import re
import ipaddress
import requests
from oal.log import configure_logging


if sys.version_info < (3,):
    import codecs

    def u(x):
        """Unicode function."""
        # pylint: disable = invalid-name
        return codecs.unicode_escape_decode(x)[0]
else:
    def u(x):
        """Unicode function."""
        # pylint: disable = invalid-name
        return x


class Office365Config(object):
    """Class for Office 365 Config."""
    # pylint: disable = too-few-public-methods

    _url = ('https://support.content.office.net'
            '/en-us/static/O365IPAddresses.xml')

    _products = ('all', 'o365', 'lyo', 'planner', 'proplus',
                 'onenote', 'spo', 'wac', 'crls', 'ex-fed',
                 'officeipad', 'exo', 'yammer', 'officemobile',
                 'sway', 'identity', 'office365video', 'rca', 'eop')

    @property
    def url(self):
        """Get url."""
        return self._url

    @property
    def products(self):
        """Get products."""
        return self._products


class Office365AddressList(object):
    """Class for Office 365 Address List."""
    # pylint: disable = too-many-instance-attributes
    # pylint: disable = no-self-use

    _oal_url = Office365Config().url
    _debug = False
    _selected_products = None
    _products = None
    minimize = False
    xml = None

    def __init__(self, debug=False):
        self.debug = debug
        self.load()

    def load(self):
        """Get xml."""
        response = requests.get(self._oal_url)
        self.xml = response.content
        self.products = ElementTree.fromstring(self.xml)
        return response

    def minimize_urls(self, addr):
        """Return minimized url."""
        root = r'(.*\.)?([a-zA-Z0-9-_]+\.[a-zA-Z]+)(\/.*)?$'
        return set([re.search(root, addy).group(2).lower() for addy in addr])

    def minimize_ips(self, addr):
        """Return minimized IP."""
        return set([ipaddr for ipaddr in ipaddress.collapse_addresses(addr)])

    def minimize_list(self, addr, addr_list):
        """Return minimized address list."""
        if 'ipv' in addr_list:
            addr = self.minimize_ips(addr)
        elif 'url' in addr_list:
            addr = self.minimize_urls(addr)
        return addr

    def address_list(self, addr_list):
        """Get sorted list of urls for all selected products."""
        addr = []
        for address_list in self.address_lists:
            if addr_list in address_list.attrib['type'].lower():
                for addy in address_list:
                    if 'ipv4' in addr_list:
                        addr.append(ipaddress.IPv4Network(u(addy.text)))
                    elif 'ipv6' in addr_list:
                        addr.append(ipaddress.IPv6Network(u(addy.text)))
                    elif 'url' in addr_list:
                        addr.append(addy.text)

        if self.minimize is True:
            addr = self.minimize_list(addr, addr_list)

        for addr in sorted(addr):
            yield addr

    def checkpoint_object(self, address):
        """Get checkpoint object."""
        obj_name = 'O365_%s' % address.network_address
        obj_comment = 'Office 365'
        obj_color = 'magenta'
        obj_ref = 'network_objects %s' % obj_name
        obj = '\n'.join([
            'create network %s' % (obj_name),
            'modify %s ipaddr %s' % (obj_ref, address.network_address),
            'modify %s netmask %s' % (obj_ref, address.netmask),
            'modify %s comments "%s"' % (obj_ref, obj_comment),
            'modify %s color "%s"' % (obj_ref, obj_color),
            'update %s' % (obj_ref)])
        return obj

    @property
    def debug(self):
        """Get debug setting."""
        return self.debug

    @debug.setter
    def debug(self, enabled):
        """Setup debugging."""
        level = 'DEBUG' if enabled else 'WARNING'
        configure_logging(level=level)
        logging.debug('Debug enabled for "%s"...', __file__)
        return level

    @property
    def selected_products(self):
        """Get selected products."""
        return self._selected_products

    @selected_products.setter
    def selected_products(self, value):
        if value:
            self._selected_products = value

    @property
    def products(self):
        """Get selected products."""
        for product in self._products:
            if product.attrib['name'].lower() in self.selected_products:
                yield product

    @products.setter
    def products(self, value):
        """Set products."""
        self._products = value

    @property
    def address_lists(self):
        """Get address lists."""
        for product in self.products:
            for address_lists in product:
                yield address_lists

    @property
    def urls(self):
        """Get urls."""
        return self.address_list('url')

    @property
    def urls_count(self):
        """Get number of urls."""
        return len([x for x in self.urls])

    @property
    def ipv4(self):
        """Get ipv4."""
        return self.address_list('ipv4')

    @property
    def ipv4_count(self):
        """Get number of ipv4."""
        return len([x for x in self.ipv4])

    @property
    def ipv6(self):
        """Get ipv6."""
        return self.address_list('ipv6')

    @property
    def ipv6_count(self):
        """Get number of ipv6."""
        return len([x for x in self.ipv6])

    @property
    def checkpoint(self):
        """Get checkpoint objects."""
        for addr in self.address_list('ipv4'):
            yield self.checkpoint_object(addr)

    @property
    def checkpoint_count(self):
        """Get number of checkpoint objects."""
        return len([x for x in self.checkpoint])

    @property
    def cisco(self):
        """Get cisco rules."""
        for addr in self.address_list('ipv4'):
            yield 'permit ip %s %s any' % (addr.network_address,
                                           addr.netmask)

    @property
    def cisco_count(self):
        """Get number of cisco rules."""
        return len([x for x in self.checkpoint])



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Logging configuration."""

from __future__ import absolute_import, unicode_literals
import logging

LEVELS = {0: 'ERROR', 1: 'WARNING', 2: 'INFO', 3: 'DEBUG'}


def configure_logging(level='WARNING'):
    """Configure root logger using a standard stream handler.

    Args:
        level (string, optional): lowest level to log to the console

    Returns:
        logging.RootLogger: root logger instance with attached handler
    """
    # get the root logger
    root_logger = logging.getLogger()
    # set the logger level to the same as will be used by the handler
    root_logger.setLevel(level)

    # customize formatter, align each column
    template = "[%(asctime)s] %(levelname)-8s %(message)s"
    formatter = logging.Formatter(template)

    # add a basic STDERR handler to the logger
    console = logging.StreamHandler()
    console.setLevel(level)
    console.setFormatter(formatter)

    root_logger.addHandler(console)
    return root_logger



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Office 365 Address Lists.

Office 365 Address Lists

:copyright: (c) 2016 by Brad Gibson
:license: MIT, see LICENSE for more details
"""
from __future__ import absolute_import, unicode_literals

__title__ = 'Office 365 Address Lists'
__summary__ = 'Office 365 Address Lists'
__uri__ = 'https://github.com/napalm255/Office 365 Address Lists'

__version__ = '0.3.0'

__author__ = 'Brad Gibson'
__email__ = 'napalm255@gmail.com'

__license__ = 'MIT'
__copyright__ = 'Copyright 2016 Brad Gibson'



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Office 365 Address Lists.

Command line interface module.
"""

from __future__ import absolute_import, print_function
import logging
import click
from oal.oal import Office365AddressList, Office365Config


@click.command()
@click.pass_context
@click.option('-d/-nd', '--debug/--no-debug', default=False,
              help='Enable or Disable Debug')
@click.option('-m/-nm', '--mini/--no-mini', default=True,
              help=('Minimize output by shortening URLs to their root '
                    'and collapsing IP networks. Enabled by default.'))
@click.option('-u/-nu', '--urls/--no-urls', default=False,
              help='Output URL List')
@click.option('-4/-n4', '--ipv4/--no-ipv4', default=False,
              help='Output IPv4 Address List')
@click.option('-6/-n6', '--ipv6/--no-ipv6', default=False,
              help='Output IPv6 Address List')
@click.option('-cp/-ncp', '--checkpoint/--no-checkpoint', default=False,
              help='Output Checkpoint dbedit script')
@click.option('-cs/-ncs', '--cisco/--no-cisco', default=False,
              help='Output Cisco ACL')
@click.argument('products', nargs=-1,
                type=click.Choice(Office365Config().products))
def cli(ctx=None, debug=False, mini=False, urls=False, ipv4=False,
        ipv6=False, checkpoint=False, cisco=False, products=None):
    """Office 365 Address Lists."""
    # pylint: disable = too-many-arguments

    def header(msg):
        """Echo."""
        line = '-'*60
        click.echo(line)
        click.echo(msg)
        click.echo(line)

    office = Office365AddressList()
    office.debug = debug
    office.minimize = mini

    logging.debug('Option --debug: %s', debug)
    logging.debug('Option --urls: %s', urls)
    logging.debug('Option --ipv4: %s', ipv4)
    logging.debug('Option --ipv6: %s', ipv6)
    logging.debug('Argument [PRODUCTS]: %s', products)

    if len(products):
        office.selected_products = products
        click.echo('Selected Products: %s\n' %
                   ', '.join(office.selected_products))
    else:
        print(ctx.get_help())
        return

    if urls is True:
        header('Unique URLs: (%s)' % (office.urls_count))
        for url in office.urls:
            click.echo(url)

    if ipv4 is True:
        header('Unique IPv4: (%s)' % (office.ipv4_count))
        for address in office.ipv4:
            click.echo(address)

    if ipv6 is True:
        header('Unique IPv6: (%s)' % (office.ipv6_count))
        for address in office.ipv6:
            click.echo(address)

    if checkpoint is True:
        header(('Unique IPv4 as a Checkpoint dbedit script: '
                '(%s)' % (office.checkpoint_count)))
        click.echo('# Save and copy output to Check Point management server')
        click.echo('# Run "dbedit -local -f /path/to/script.txt"')
        for address in office.checkpoint:
            click.echo(address)
        click.echo('update_all')
        click.echo('savedb')

    if cisco is True:
        header(('Unique IPv4 as a ' 'Cisco ACL: (%s)' % (office.cisco_count)))
        for address in office.cisco:
            click.echo(address)


def main():
    """Main entry point."""
    click.echo('='*60)
    click.echo('Office 365 Address Lists')
    click.echo('='*60)
    cli()


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `oal` module."""

import pytest
from contextlib import contextmanager
from click.testing import CliRunner
from oal.__main__ import cli


class TestOal(object):

    @classmethod
    def setup_class(cls):
        pass

    def test_command_line_interface(self):
        """Test command line interface."""
        runner = CliRunner()
        result = runner.invoke(cli)
        assert result.exit_code == 0
        assert 'Office 365' in result.output

    def test_help(self):
        """Test help."""
        runner = CliRunner()
        help_result = runner.invoke(cli, ['--help'])
        assert help_result.exit_code == 0
        assert 'Show this message and exit.' in help_result.output

    def test_options(self):
        """Test options."""
        runner = CliRunner()
        product_url_result = runner.invoke(cli, ['-u', 'o365'])
        assert product_url_result.exit_code == 0
        assert 'Selected Products: o365' in product_url_result.output
        product_v4_result = runner.invoke(cli, ['-4', 'o365'])
        assert product_v4_result.exit_code == 0
        assert 'Selected Products: o365' in product_v4_result.output
        product_v6_result = runner.invoke(cli, ['-6', 'eop'])
        assert product_v6_result.exit_code == 0
        assert 'Selected Products: eop' in product_v6_result.output

    @classmethod
    def teardown_class(cls):
        pass


if __name__ == '__main__':
    sys.exit(pytest.main())
